# Copyright 2014, 2017 IBM Corp.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import mock
import testtools

from pypowervm.tasks.monitor import host_cpu
import pypowervm.tests.test_fixtures as pvm_fx

LOG = logging.getLogger(__name__)


class TestHostCPUBase(testtools.TestCase):

    def setUp(self):
        super(TestHostCPUBase, self).setUp()

        # Fixture for the adapter
        self.adpt = self.useFixture(pvm_fx.AdapterFx()).adpt
        ensure_ltm_p = mock.patch(
            'pypowervm.tasks.monitor.util.ensure_ltm_monitors')
        refresh_p = mock.patch(
            'pypowervm.tasks.monitor.util.MetricCache._refresh_if_needed')
        self.mock_ensure_ltm_monitors = ensure_ltm_p.start()
        self.mock_refresh_if_needed = refresh_p.start()
        self.addCleanup(ensure_ltm_p.stop)
        self.addCleanup(refresh_p.stop)


class TestHostCPUFreq(TestHostCPUBase):

    def test_get_cpu_freq(self):
        # _get_cpu_freq() should return an int based on the clock line of the
        # file
        m = mock.mock_open(
            read_data='processor : 12\nclock : 4116.000000MHz\n')
        m.return_value.__iter__ = lambda self: iter(self.readline, '')
        with mock.patch('pypowervm.tasks.monitor.host_cpu.open', m):
            host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        self.assertEqual(host_stats.cpu_freq, 4116)


class TestHostCPUMetricCache(TestHostCPUBase):

    def setUp(self):
        super(TestHostCPUMetricCache, self).setUp()
        get_cpu_freq_p = mock.patch('pypowervm.tasks.monitor.host_cpu.'
                                    'HostCPUMetricCache._get_cpu_freq')
        self.mock_get_cpu_freq = get_cpu_freq_p.start()
        self.addCleanup(get_cpu_freq_p.stop)

    def _get_sample(self, lpar_id, sample):
        for lpar in sample.lpars:
            if lpar.id == lpar_id:
                return lpar
        return None

    def test_refresh(self):
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        host_stats.refresh()
        # Called once in init and once in refesh()
        self.assertEqual(self.mock_refresh_if_needed.call_count, 2)

    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_get_fw_cycles_delta')
    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_get_total_cycles_delta')
    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_gather_user_cycles_delta')
    def test_update_internal_metric(self, mock_user_cycles, mock_total_cycles,
                                    mock_fw_cycles):

        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')

        # Make sure totals are 0 if there is no data.
        host_stats.cur_phyp = None
        host_stats._update_internal_metric()
        self.assertEqual(host_stats.total_cycles, 0)
        self.assertEqual(host_stats.total_user_cycles, 0)
        self.assertEqual(host_stats.total_fw_cycles, 0)

        # Create mock phyp objects to test with
        mock_phyp = mock.MagicMock()
        mock_fw_cycles.return_value = 58599310268
        mock_prev_phyp = mock.MagicMock()

        # Mock methods not currently under test
        mock_user_cycles.return_value = 50
        mock_total_cycles.return_value = 1.6125945178663e+16

        # Make the 'prev' the current...for the first pass
        host_stats.cur_phyp = mock_prev_phyp
        host_stats.prev_phyp = None
        host_stats._update_internal_metric()

        self.assertEqual(host_stats.total_cycles, 1.6125945178663e+16)
        self.assertEqual(host_stats.total_user_cycles, 50)
        self.assertEqual(host_stats.total_fw_cycles, 58599310268)

        # Mock methods not currently under test
        mock_user_cycles.return_value = 30010090000
        mock_total_cycles.return_value = 1.6125945178663e+16

        # Now 'increment' it with a new current/previous
        host_stats.cur_phyp = mock_phyp
        host_stats.prev_phyp = mock_prev_phyp
        mock_user_cycles.return_value = 100000
        host_stats._update_internal_metric()

        # Validate the new values. Note that these values are 'higher' because
        # they are running totals.
        new_fw = 58599310268 * 2
        new_total = 1.6125945178663e+16 * 2
        self.assertEqual(host_stats.total_cycles, new_total)
        self.assertEqual(host_stats.total_user_cycles, 100050)
        self.assertEqual(host_stats.total_fw_cycles, new_fw)

    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_get_fw_cycles_delta')
    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_get_total_cycles_delta')
    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_gather_user_cycles_delta')
    def test_update_internal_metric_bad_total(
            self, mock_user_cycles, mock_tot_cycles, mock_fw_cycles):
        """Validates that if the total cycles are off, we handle."""
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_user_cycles.return_value = 30010090000
        mock_fw_cycles.return_value = 58599310268

        # Mock the total cycles to some really low number.
        mock_tot_cycles.return_value = 5

        # Create mock phyp objects to test with
        mock_phyp = mock.MagicMock()
        mock_prev_phyp = mock.MagicMock()
        mock_phyp.sample.system_firmware.utilized_proc_cycles = 58599310268

        # Run the actual test - 'increment' it with a new current/previous
        host_stats.cur_phyp = mock_phyp
        host_stats.prev_phyp = mock_prev_phyp
        host_stats._update_internal_metric()

        # Validate the results. The total cycles are set to the sum of user
        # and fw when the total is bad.
        self.assertEqual(host_stats.total_cycles, 88609400268)
        self.assertEqual(host_stats.total_user_cycles, 30010090000)
        self.assertEqual(host_stats.total_fw_cycles, 58599310268)

    @mock.patch('pypowervm.tasks.monitor.host_cpu.HostCPUMetricCache.'
                '_delta_proc_cycles')
    def test_gather_user_cycles_delta(self, mock_cycles):
        # Crete objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_phyp = mock.MagicMock()
        mock_prev_phyp = mock.MagicMock()

        # Mock methods not currently under test
        mock_cycles.return_value = 15005045000

        # Test that we can run with previous samples and then without.
        host_stats.cur_phyp = mock_phyp
        host_stats.prev_phyp = mock_prev_phyp
        resp = host_stats._gather_user_cycles_delta()
        self.assertEqual(30010090000, resp)

        # Now test if there is no previous sample.  Since there are no previous
        # samples, it will be 0.
        host_stats.prev_phyp = None
        mock_cycles.return_value = 0
        resp = host_stats._gather_user_cycles_delta()
        self.assertEqual(0, resp)

    def test_delta_proc_cycles(self):
        # Create objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_phyp, mock_prev_phyp = self._get_mock_phyps()

        # Test that a previous sample allows us to gather the delta across all
        # of the VMs.  This should take into account the scenario where a LPAR
        # is deleted and a new one takes its place (LPAR ID 6)
        delta = host_stats._delta_proc_cycles(mock_phyp.sample.lpars,
                                              mock_prev_phyp.sample.lpars)
        self.assertEqual(10010000000, delta)

        # Now test as if there is no previous data.  This results in 0 as they
        # could have all been LPMs with months of cycles (rather than 30
        # seconds delta).
        delta2 = host_stats._delta_proc_cycles(mock_phyp.sample.lpars, None)
        self.assertEqual(0, delta2)
        self.assertNotEqual(delta2, delta)

        # Test that if previous sample had 0 values, the sample is not
        # considered for evaluation, and resultant delta cycles is 0.
        prev_lpar_sample = mock_prev_phyp.sample.lpars[0].processor
        prev_lpar_sample.util_cap_proc_cycles = 0
        prev_lpar_sample.util_uncap_proc_cycles = 0
        prev_lpar_sample.idle_proc_cycles = 0
        delta3 = host_stats._delta_proc_cycles(mock_phyp.sample.lpars,
                                               mock_prev_phyp.sample.lpars)
        self.assertEqual(0, delta3)

    def test_delta_user_cycles(self):
        # Create objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_phyp, mock_prev_phyp = self._get_mock_phyps()
        mock_phyp.sample.lpars[0].processor.util_cap_proc_cycles = 250000
        mock_phyp.sample.lpars[0].processor.util_uncap_proc_cycles = 250000
        mock_phyp.sample.lpars[0].processor.idle_proc_cycles = 500
        mock_prev_phyp.sample.lpars[0].processor.util_cap_proc_cycles = 0
        num = 455000
        mock_prev_phyp.sample.lpars[0].processor.util_uncap_proc_cycles = num
        mock_prev_phyp.sample.lpars[0].processor.idle_proc_cycles = 1000

        # Test that a previous sample allows us to gather just the delta.
        new_elem = self._get_sample(4, mock_phyp.sample)
        old_elem = self._get_sample(4, mock_prev_phyp.sample)
        delta = host_stats._delta_user_cycles(new_elem, old_elem)
        self.assertEqual(45500, delta)

        # Validate the scenario where we don't have a previous.  Should default
        # to 0, given no context of why the previous sample did not have the
        # data.
        delta = host_stats._delta_user_cycles(new_elem, None)
        self.assertEqual(0, delta)

    def test_find_prev_sample(self):
        # Create objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_lpar_4A = mock.Mock()
        mock_lpar_4A.configure_mock(id=4, name='A')
        mock_lpar_4A.processor = mock.MagicMock(
            entitled_proc_cycles=500000)
        mock_lpar_6A = mock.Mock()
        mock_lpar_6A.configure_mock(id=6, name='A')
        mock_lpar_6B = mock.Mock()
        mock_lpar_6B.configure_mock(id=6, name='B')
        mock_phyp = mock.MagicMock(sample=mock.MagicMock(lpars=[mock_lpar_4A,
                                                                mock_lpar_6A]))
        mock_prev_phyp = mock.MagicMock(sample=mock.MagicMock(
            lpars=[mock_lpar_4A, mock_lpar_6B]))

        # Sample 6 in the current shouldn't match the previous.  It has the
        # same LPAR ID, but a different name.  This is considered different
        new_elem = self._get_sample(6, mock_phyp.sample)
        prev = host_stats._find_prev_sample(new_elem,
                                            mock_prev_phyp.sample.lpars)
        self.assertIsNone(prev)

        # Lpar 4 should be in the old one.  Match that up.
        new_elem = self._get_sample(4, mock_phyp.sample)
        prev = host_stats._find_prev_sample(new_elem,
                                            mock_prev_phyp.sample.lpars)
        self.assertIsNotNone(prev)
        self.assertEqual(500000, prev.processor.entitled_proc_cycles)

        # Test that we get None back if there are no previous samples
        prev = host_stats._find_prev_sample(new_elem, None)
        self.assertIsNone(prev)

    def test_get_total_cycles(self):
        # Mock objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')
        mock_phyp = mock.MagicMock()
        mock_phyp.sample = mock.MagicMock()
        mock_phyp.sample.processor.configurable_proc_units = 5
        mock_phyp.sample.time_based_cycles = 500
        host_stats.cur_phyp = mock_phyp

        # Make sure we get the full system cycles.
        max_cycles = host_stats._get_total_cycles_delta()
        self.assertEqual(2500, max_cycles)

    def test_get_total_cycles_diff_cores(self):
        # Mock objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')

        # Latest Sample
        mock_phyp = mock.MagicMock(sample=mock.MagicMock())
        mock_phyp.sample.processor.configurable_proc_units = 48
        mock_phyp.sample.time_based_cycles = 1000
        host_stats.cur_phyp = mock_phyp

        # Earlier sample.  Use a higher proc unit sample
        mock_phyp = mock.MagicMock(sample=mock.MagicMock())
        mock_phyp.sample.processor.configurable_proc_units = 1
        mock_phyp.sample.time_based_cycles = 500
        host_stats.prev_phyp = mock_phyp

        # Make sure we get the full system cycles.
        max_cycles = host_stats._get_total_cycles_delta()
        self.assertEqual(24000, max_cycles)

    def test_get_firmware_cycles(self):
        # Mock objects to test with
        host_stats = host_cpu.HostCPUMetricCache(self.adpt, 'host_uuid')

        # Latest Sample
        mock_phyp = mock.MagicMock(sample=mock.MagicMock())
        mock_phyp.sample.system_firmware.utilized_proc_cycles = 2000
        # Previous Sample
        prev_phyp = mock.MagicMock(sample=mock.MagicMock())
        prev_phyp.sample.system_firmware.utilized_proc_cycles = 1000

        host_stats.cur_phyp = mock_phyp
        host_stats.prev_phyp = prev_phyp
        # Get delta
        delta_firmware_cycles = host_stats._get_fw_cycles_delta()
        self.assertEqual(1000, delta_firmware_cycles)

    def _get_mock_phyps(self):
        """Helper method to return cur_phyp and prev_phyp."""
        mock_lpar_4A = mock.Mock()
        mock_lpar_4A.configure_mock(id=4, name='A')
        mock_lpar_4A.processor = mock.MagicMock(
            util_cap_proc_cycles=5005045000,
            util_uncap_proc_cycles=5005045000,
            idle_proc_cycles=10000)
        mock_lpar_4A_prev = mock.Mock()
        mock_lpar_4A_prev.configure_mock(id=4, name='A')
        mock_lpar_4A_prev.processor = mock.MagicMock(
            util_cap_proc_cycles=40000,
            util_uncap_proc_cycles=40000,
            idle_proc_cycles=0)
        mock_phyp = mock.MagicMock(sample=mock.MagicMock(lpars=[mock_lpar_4A]))
        mock_prev_phyp = mock.MagicMock(
            sample=mock.MagicMock(lpars=[mock_lpar_4A_prev]))
        return mock_phyp, mock_prev_phyp



# Copyright 2017 IBM Corp.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import testtools

import pypowervm.entities as ent
from pypowervm.tasks.hdisk import _rbd as rbd
import pypowervm.tests.test_fixtures as fx
from pypowervm.wrappers import job


class TestRbd(testtools.TestCase):

    def setUp(self):
        super(TestRbd, self).setUp()
        entry = ent.Entry({}, ent.Element('Dummy', None), None)
        self.mock_job = job.Job(entry)
        self.adpt = self.useFixture(fx.AdapterFx()).adpt

    @mock.patch('pypowervm.wrappers.job.Job.create_job_parameter')
    @mock.patch('pypowervm.wrappers.job.Job.wrap')
    @mock.patch('pypowervm.wrappers.job.Job.run_job')
    @mock.patch('pypowervm.wrappers.job.Job.get_job_results_as_dict')
    def test_rbd_exists(self, mock_job_res, mock_run_job, mock_job_w,
                        mock_job_p):
        mock_job_w.return_value = self.mock_job
        mock_uuid = 'uuid'
        mock_name = 'pool/image'
        mock_parm = mock.MagicMock()
        mock_job_p.return_value = mock_parm
        args = ['VirtualIOServer', mock_uuid]
        kwargs = {'suffix_type': 'do', 'suffix_parm': ('RBDExists')}
        mock_job_res.return_value = {'exists': 'true'}
        self.assertTrue(rbd.rbd_exists(self.adpt, mock_uuid, mock_name))

        self.adpt.read.assert_called_once_with(*args, **kwargs)
        mock_run_job.assert_called_once_with(mock_uuid, job_parms=[mock_parm],
                                             timeout=120)
        mock_job_p.assert_any_call('name', mock_name)
        self.assertEqual(1, mock_run_job.call_count)
        mock_job_res.return_value = {'exists': 'false'}
        mock_job_p.return_value = mock_parm
        self.assertFalse(rbd.rbd_exists(self.adpt, mock_uuid, mock_name))



# Copyright 2016, 2017 IBM Corp.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import ast
import six

from oslo_log import log as logging

import pypowervm.const as c
from pypowervm import exceptions as pexc
from pypowervm.wrappers import job
from pypowervm.wrappers.virtual_io_server import VIOS


LOG = logging.getLogger(__name__)
_JOB_NAME = "ISCSIDiscovery"
_ISCSI_REMOVE = "ISCSIRemove"


class ISCSIStatus(object):
    """ISCSI status codes."""
    ISCSI_SUCCESS = '0'
    ISCSI_ERR = '1'
    ISCSI_ERR_SESS_NOT_FOUND = '3'
    ISCSI_ERR_LOGIN = '5'
    ISCSI_ERR_INVAL = '7'
    ISCSI_ERR_TRANS_TIMEOUT = '8'
    ISCSI_ERR_INTERNAL = '9'
    ISCSI_ERR_LOGOUT = '10'
    ISCSI_ERR_SESS_EXISTS = '15'
    ISCSI_ERR_NO_OBJS_FOUND = '21'
    ISCSI_ERR_HOST_NOT_FOUND = '23'
    ISCSI_ERR_LOGIN_AUTH_FAILED = '24'
    ISCSI_COMMAND_NOT_FOUND = '127'


_GOOD_DISCOVERY_STATUSES = [ISCSIStatus.ISCSI_SUCCESS,
                            ISCSIStatus.ISCSI_ERR_SESS_EXISTS]
_GOOD_REMOVE_STATUSES = [ISCSIStatus.ISCSI_SUCCESS,
                         ISCSIStatus.ISCSI_ERR_NO_OBJS_FOUND]


def _find_dev_by_iqn(cmd_output, iqn, host_ip):
    """Find device name and udid corresponding to an IQN

    The iqn parameter can be a singular iqn or a list of iqns. If a list of
    iqns is given, we can return the device name and udid for any of the iqns,
    since this implies a multipath device which would have the same return
    values for all iqns.

    :param cmd_output: A list of "iqn device_name udid"
    :param host_ip: The portal or list of portals for the iscsi target. A
                    portal looks like ip:port.
    :param iqn: The IQN (iSCSI Qualified Name) or list of IQNs for the created
                volume on the target (e.g. iqn.2016-06.world.srv:target00).
    :return: The device name of the created volume.
    :return: The UniqueDeviceId of the create volume.
    """
    for dev in cmd_output:
        try:
            outiqn, outname, udid = dev.split()
            if ((isinstance(iqn, six.string_types) and outiqn == iqn) or
                    outiqn in iqn):
                return outname, udid
        except ValueError:
            LOG.warning("Invalid device output: %(dev)s" % {'dev': dev})
            continue

    LOG.error("Expected IQN %(IQN)s not found on iscsi target %(host_ip)s" %
              {'IQN': iqn, 'host_ip': host_ip})
    return None, None


def _add_parameter(job_parms, name, value):
    """Adds key/value to job parameter list

    Checks for null value and does any conversion needed for value to be a
    string

    :param job_parms: List of parameters for the job which will be updated by
                      this method.
    :param key: Parameter name
    :param value: Parameter value
    """
    if value or value == 0 or value is False:
        if isinstance(value, (six.string_types, int, bool)):
            value = six.text_type(value)
        else:
            value = '[' + ','.join([str(val) for val in value]) + ']'
        job_parms.append(job.Job.create_job_parameter(name, value))


def discover_iscsi(adapter, host_ip, user, password, iqn, vios_uuid,
                   transport_type=None, lunid=None, iface_name=None, auth=None,
                   discovery_auth=None, discovery_username=None,
                   discovery_password=None, multipath=False):
    """Runs iscsi discovery and login job

    :param adapter: pypowervm adapter
    :param host_ip: The portal or list of portals for the iscsi target. A
                    portal looks like ip:port.
    :param user: The username needed for authentication.
    :param password: The password needed for authentication.
    :param iqn: The IQN (iSCSI Qualified Name) or list of IQNs for the created
                volume on the target (e.g. iqn.2016-06.world.srv:target00).
    :param vios_uuid: The uuid of the VIOS (VIOS must be a Novalink VIOS type).
    :param transport_type: (Deprecated) Transport type of the volume to be
                           connected. Use iface_name instead.
    :param lunid: Target LUN ID or list of LUN IDs for the volume.
    :param iface_name: Iscsi iface name to use for the connection.
    :param auth: Authentication type
    :param discovery_auth: Discovery authentication type.
    :param discovery_username: The username needed for discovery
                               authentication.
    :param discovery_password: The password needed for discovery
                               authentication.
    :param multipath: Whether the connection is multipath or not.
    :return: The device name of the created volume.
    :return: The UniqueDeviceId of the create volume.
    :raise: ISCSIDiscoveryFailed in case of bad return code.
    :raise: JobRequestFailed in case of failure.
    """

    resp = adapter.read(VIOS.schema_type, vios_uuid,
                        suffix_type=c.SUFFIX_TYPE_DO, suffix_parm=(_JOB_NAME))
    job_wrapper = job.Job.wrap(resp)

    # Create job parameters
    job_parms = []
    _add_parameter(job_parms, 'auth', auth)
    _add_parameter(job_parms, 'user', user)
    _add_parameter(job_parms, 'password', password)
    _add_parameter(job_parms, 'ifaceName', iface_name or transport_type)
    _add_parameter(job_parms, 'targetIQN', iqn)
    _add_parameter(job_parms, 'hostIP', host_ip)
    _add_parameter(job_parms, 'targetLUN', lunid)
    _add_parameter(job_parms, 'multipath', multipath)
    if multipath:
        _add_parameter(job_parms, 'discoveryAuth', discovery_auth)
        _add_parameter(job_parms, 'discoveryUser',
                       discovery_username)
        _add_parameter(job_parms, 'discoveryPassword',
                       discovery_password)

    try:
        job_wrapper.run_job(vios_uuid, job_parms=job_parms, timeout=120)
    except pexc.JobRequestFailed:
        results = job_wrapper.get_job_results_as_dict()
        # Ignore if the command is performed on NotSupported AIX VIOS
        if results.get('RETURN_CODE') != ISCSIStatus.ISCSI_COMMAND_NOT_FOUND:
            raise
        return None, None

    results = job_wrapper.get_job_results_as_dict()

    # RETURN_CODE: for iscsiadm status
    status = results.get('RETURN_CODE')
    if status not in _GOOD_DISCOVERY_STATUSES:
        raise pexc.ISCSIDiscoveryFailed(vios_uuid=vios_uuid, status=status)

    # DEV_OUTPUT: ["IQN1 dev1 udid", "IQN2 dev2 udid"]
    output = ast.literal_eval(results.get('DEV_OUTPUT'))

    # Find dev corresponding to given IQN
    return _find_dev_by_iqn(output, iqn, host_ip)


def discover_iscsi_initiator(adapter, vios_uuid):
    """Discovers the initiator name.

    :param adapter: pypowervm adapter
    :param vios_uuid: The uuid of the VIOS (VIOS must be a Novalink VIOS type).
    :return: The iscsi initiator name.
    :raise: ISCSIDiscoveryFailed in case of failure.
    """
    resp = adapter.read(VIOS.schema_type, vios_uuid,
                        suffix_type=c.SUFFIX_TYPE_DO, suffix_parm=(_JOB_NAME))
    job_wrapper = job.Job.wrap(resp)

    job_wrapper.run_job(vios_uuid, timeout=120)
    results = job_wrapper.get_job_results_as_dict()

    # process iscsi return code.
    status = results.get('RETURN_CODE')
    if status not in _GOOD_DISCOVERY_STATUSES:
        raise pexc.ISCSIDiscoveryFailed(vios_uuid=vios_uuid, status=status)

    # InitiatorName: iqn.2010-10.org.openstack:volume-4a75e9f7-dfa3
    return results.get('InitiatorName')


def remove_iscsi(adapter, targetIQN, vios_uuid, iface_name=None, lun=None,
                 portal=None, multipath=False):
    """Remove an iSCSI lun from a session.

    If the last lun was removed from the session, also logout of the session.
    The iSCSI volume with the given targetIQN must not have any mappings from
    the VIOS to a client when this is called.

    :param adapter: pypowervm adapter
    :param targetIQN: The IQN (iSCSI Qualified Name) or list of IQNs for the
                      created volume on the target.
                      (e.g. iqn.2016-06.world.srv:target00)
    :param vios_uuid: The uuid of the VIOS (VIOS must be a Novalink VIOS type).
    :param iface_name: Name of the iface used for the connection.
    :param lun: The lun or list of luns to be removed.
    :param portal: The portal or list of portals associated with the created
                   volume (ip:port).
    :param multipath: Whether the connection is multipath or not.
    :raise: ISCSIRemoveFailed in case of bad return code.
    :raise: JobRequestFailed in case of failure.
    """
    resp = adapter.read(VIOS.schema_type, vios_uuid,
                        suffix_type=c.SUFFIX_TYPE_DO,
                        suffix_parm=(_ISCSI_REMOVE))
    job_wrapper = job.Job.wrap(resp)
    job_parms = []
    _add_parameter(job_parms, 'ifaceName', iface_name)
    _add_parameter(job_parms, 'targetIQN', targetIQN)
    _add_parameter(job_parms, 'targetPORTAL', portal)
    _add_parameter(job_parms, 'targetLUN', lun)
    _add_parameter(job_parms, 'multipath', multipath)
    try:
        job_wrapper.run_job(vios_uuid, job_parms=job_parms, timeout=120)
    except pexc.JobRequestFailed:
        results = job_wrapper.get_job_results_as_dict()
        # Ignore if the command is performed on NotSupported AIX VIOS
        if results.get('RETURN_CODE') != ISCSIStatus.ISCSI_COMMAND_NOT_FOUND:
            raise
        return
    results = job_wrapper.get_job_results_as_dict()
    status = results.get('RETURN_CODE')
    if status not in _GOOD_REMOVE_STATUSES:
        raise pexc.ISCSIRemoveFailed(vios_uuid=vios_uuid, status=status)



# Copyright 2017 IBM Corp.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging

import pypowervm.const as c
from pypowervm.wrappers import job
from pypowervm.wrappers.virtual_io_server import VIOS

LOG = logging.getLogger(__name__)
_JOB_NAME = "RBDExists"


def rbd_exists(adapter, vios_uuid, name):
    """Check if rbd exists on vios

    :param adapter: pypowervm adapter
    :param vios_uuid: The uuid of the VIOS (VIOS must be a Novalink VIOS type).
    :param name: Name of the rbd volume (pool/image)
    :return: The device name of the created volume.
    """

    resp = adapter.read(VIOS.schema_type, vios_uuid,
                        suffix_type=c.SUFFIX_TYPE_DO, suffix_parm=(_JOB_NAME))
    job_wrapper = job.Job.wrap(resp)

    # Create job parameters
    job_parms = [job_wrapper.create_job_parameter('name', name)]
    job_wrapper.run_job(vios_uuid, job_parms=job_parms, timeout=120)
    results = job_wrapper.get_job_results_as_dict()

    return True if results.get('exists') == "true" else False
